from tensorflow.keras import Sequential
import tensorflow.keras as keras
import tensorflow.keras.layers as layers

from harmonic.benchmark import run_n
from data.mnist import get_mnist_rot, get_mnist


def create_cnn():
    model = Sequential([
        layers.Reshape((28, 28, 1)),
        layers.Conv2D(16, (3, 3), activation='relu', padding='SAME'),  # Input: 28x28x1
        layers.Conv2D(16, (3, 3), activation='relu', padding='SAME'),  # Input: 28x28x16
        layers.BatchNormalization(),
        layers.MaxPooling2D((2, 2)),
        layers.Conv2D(16, (3, 3), activation='relu', padding='SAME'),  # Input: 14x14x16
        layers.Conv2D(16, (3, 3), activation='relu', padding='SAME'),  # Input: 14x14x16
        layers.Flatten(),
        layers.Dense(10)
    ])

    model.compile(optimizer='adam', loss=keras.losses.SparseCategoricalCrossentropy(from_logits=True),
                  metrics=['accuracy'])
    return model


if __name__ == '__main__':
    _, test = get_mnist_rot()
    train, _ = get_mnist()
    model = create_cnn()
    run_n(model, train, test)
